<?php
/* *
 * 彩虹易支付SDK服务类
 * 说明：
 * 包含发起支付、查询订单、回调验证等功能
 */

namespace HaoZiTeam\WPChatGPT\Inc;

class EpayCore {
	private $pid;

	private $key;

	private string $submit_url;

	private string $mapi_url;

	private string $api_url;

	private string $sign_type = 'MD5';

	public function __construct( $config ) {
		$this->pid        = $config['pid'];
		$this->key        = $config['key'];
		$this->submit_url = $config['apiurl'] . 'submit.php';
		$this->mapi_url   = $config['apiurl'] . '/pay/apisubmit';
		$this->api_url    = $config['apiurl'] . 'api.php';
	}

	// 发起支付（页面跳转）
	public function pagePay( $param_tmp, $button = '正在跳转' ): string {
		$param = $this->buildRequestParam( $param_tmp );

		$html = '<form id="dopay" action="' . $this->submit_url . '" method="post">';
		foreach ( $param as $k => $v ) {
			$html .= '<input type="hidden" name="' . $k . '" value="' . $v . '"/>';
		}
		$html .= '<input type="submit" value="' . $button . '"></form><script>document.getElementById("dopay").submit();</script>';

		return $html;
	}

	// 发起支付（获取链接）
	public function getPayLink( $param_tmp ): string {
		$param = $this->buildRequestParam( $param_tmp );

		return $this->submit_url . '?' . http_build_query( $param );
	}

	// 发起支付（API接口）
	public function apiPay( $param_tmp ) {
		$param    = $this->buildRequestParam( $param_tmp );
		$response = $this->getHttpResponse( $this->mapi_url, http_build_query( $param ) );

		return json_decode( $response, true );
	}

	// 异步回调验证
	public function verifyNotify(): bool {
		if ( empty( $_GET ) ) {
			return false;
		}

		$sign = $this->getSign( $_GET );

		if ( $sign === $_GET['sign'] ) {
			$signResult = true;
		} else {
			$signResult = false;
		}

		return $signResult;
	}

	// 同步回调验证
	public function verifyReturn(): bool {
		if ( empty( $_GET ) ) {
			return false;
		}

		$sign = $this->getSign( $_GET );

		if ( $sign === $_GET['sign'] ) {
			$signResult = true;
		} else {
			$signResult = false;
		}

		return $signResult;
	}

	// 查询订单支付状态
	public function orderStatus( $trade_no ): bool {
		$result = $this->queryOrder( $trade_no );
		if ( $result['status'] == 1 ) {
			return true;
		} else {
			return false;
		}
	}

	// 查询订单
	public function queryOrder( $trade_no ) {
		$url      = $this->api_url . '?act=order&pid=' . $this->pid . '&key=' . $this->key . '&trade_no=' . $trade_no;
		$response = $this->getHttpResponse( $url );

		return json_decode( $response, true );
	}

	private function buildRequestParam( $param ) {
		$sign               = $this->getSign( $param );
		$param['sign']      = $sign;
		$param['sign_type'] = $this->sign_type;

		return $param;
	}

	// 计算签名
	private function getSign( $param ): string {
		ksort( $param );
		$signStr = '';

		foreach ( $param as $k => $v ) {
			if ( $k != 'sign' && $k != 'sign_type' && $k != 'user_id' && $k != 'pay_type' && $v != '' ) {
				$signStr .= $k . '=' . $v . '&';
			}
		}
		$signStr = substr( $signStr, 0, - 1 );
		$signStr .= $this->key;

		return md5( $signStr );
	}

	// 请求外部资源
	private function getHttpResponse( $url, $post = false ) {
		$ch = curl_init( $url );
		curl_setopt( $ch, CURLOPT_TIMEOUT, 10 );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, false );
		$httpHeader[] = 'Accept: */*';
		$httpHeader[] = 'Accept-Language: zh-CN,zh;q=0.8';
		$httpHeader[] = 'Connection: close';
		curl_setopt( $ch, CURLOPT_HTTPHEADER, $httpHeader );
		curl_setopt( $ch, CURLOPT_HEADER, false );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
		if ( $post ) {
			curl_setopt( $ch, CURLOPT_POST, true );
			curl_setopt( $ch, CURLOPT_POSTFIELDS, $post );
		}
		$response = curl_exec( $ch );
		curl_close( $ch );

		return $response;
	}
}
