<?php

namespace HaoZiTeam\WPChatGPT\Inc;

use Exception;

class XunHuPay {
	private string $url;
	private string $appid;
	private string $appsecret;
	private string $plugins = 'wp-chatgpt';

	public function __construct( $appid, $appsecret, $url = 'https://api.xunhupay.com/payment/do.html' ) {
		$this->appid     = $appid;
		$this->appsecret = $appsecret;
		$this->url       = $url;
	}

	/**
	 * @throws Exception
	 */
	public static function http_post( $url, $data ) {
		if ( ! function_exists( 'curl_init' ) ) {
			throw new Exception( 'php未安装curl组件', 500 );
		}

		$protocol = ( ! empty ( $_SERVER ['HTTPS'] ) && $_SERVER ['HTTPS'] !== 'off' || $_SERVER ['SERVER_PORT'] == 443 ) ? "https://" : "http://";
		$siteurl  = $protocol . $_SERVER['HTTP_HOST'];

		$ch = curl_init();
		curl_setopt( $ch, CURLOPT_TIMEOUT, 60 );
		curl_setopt( $ch, CURLOPT_URL, $url );
		curl_setopt( $ch, CURLOPT_REFERER, $siteurl );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, false );
		curl_setopt( $ch, CURLOPT_HEADER, false );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_POST, true );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $data );
		$response       = curl_exec( $ch );
		$httpStatusCode = curl_getinfo( $ch, CURLINFO_HTTP_CODE );
		$error          = curl_error( $ch );
		curl_close( $ch );
		if ( $httpStatusCode != 200 ) {
			throw new Exception( "invalid httpstatus:$httpStatusCode ,response:$response,detail_error:" . $error, $httpStatusCode );
		}

		return $response;
	}

	public static function generate_xh_hash( array $datas, $hashkey ) {
		ksort( $datas );
		reset( $datas );

		$pre = array();
		foreach ( $datas as $key => $data ) {
			if ( is_null( $data ) || $data === '' ) {
				continue;
			}
			if ( $key == 'hash' ) {
				continue;
			}
			$pre[ $key ] = stripslashes( $data );
		}

		$arg   = '';
		$qty   = count( $pre );
		$index = 0;

		foreach ( $pre as $key => $val ) {
			$arg .= "$key=$val";
			if ( $index ++ < ( $qty - 1 ) ) {
				$arg .= "&";
			}
		}

		return md5( $arg . $hashkey );
	}

	/**
	 * 获取支付链接
	 */
	public function getPayUrl( $data ): array {
		$default_data = array(
			'version'        => '1.1',//固定值，api 版本，目前暂时是1.1
			'lang'           => 'zh-cn', //必须的，zh-cn或en-us 或其他，根据语言显示页面
			'plugins'        => $this->plugins,//必须的，根据自己需要自定义插件ID，唯一的，匹配[a-zA-Z\d\-_]+
			'appid'          => $this->appid, //必须的，APPID
			'trade_order_id' => uniqid(), //必须的，网站订单ID，唯一的，匹配[a-zA-Z\d\-_]+
			'payment'        => 'wechat',//必须的，支付接口标识：wechat(微信接口)|alipay(支付宝接口)
			'type'           => 'WAP',//固定值"WAP" H5支付必填
			'wap_url'        => home_url(),//网站域名，H5支付必填
			'wap_name'       => get_bloginfo( 'name' ),//网站名称，H5支付必填
			'total_fee'      => '0.1',//人民币，单位精确到分(测试账户只支持0.1元内付款)
			'title'          => '耐克球鞋', //必须的，订单标题，长度32或以内
			'time'           => time(),//必须的，当前时间戳，根据此字段判断订单请求是否已超时，防止第三方攻击服务器
			'notify_url'     => 'https://www.xx.com/notify.php', //必须的，支付成功异步回调接口
			'nonce_str'      => uniqid()//必须的，随机字符串，作用：1.避免服务器缓存，2.防止安全密钥被猜测出来
		);
		// 合并数组
		$data         = array_merge( $default_data, $data );
		$data['hash'] = self::generate_xh_hash( $data, $this->appsecret );
		try {
			$response = self::http_post( $this->url, json_encode( $data ) );
			$result   = $response ? json_decode( $response, true ) : null;
			if ( ! $result ) {
				throw new Exception( 'Internal server error', 500 );
			}

			$hash = self::generate_xh_hash( $result, $this->appsecret );
			if ( ! isset( $result['hash'] ) || $hash != $result['hash'] ) {
				throw new Exception( 'Invalid sign!', 40029 );
			}

			if ( $result['errcode'] != 0 ) {
				throw new Exception( $result['errmsg'], $result['errcode'] );
			}

			return [
				'status' => true,
				'url'    => $result['url'],
			];
		} catch ( Exception $e ) {
			return [
				'status'  => false,
				'message' => $e->getMessage(),
			];
		}
	}

	/**
	 * 支付回调验证
	 */
	public function notifyCheck(): bool {
		/**
		 * 回调数据
		 * array(
		 * 'trade_order_id'，商户网站订单ID
		 * 'total_fee',订单支付金额
		 * 'transaction_id',//支付平台订单ID
		 * 'order_date',//支付时间
		 * 'plugins',//自定义插件ID,与支付请求时一致
		 * 'status'=>'OD'//订单状态，OD已支付，WP未支付
		 *   )
		 */
		$data = $_POST;
		foreach ( $data as $k => $v ) {
			$data[ $k ] = stripslashes( $v );
		}

		if ( ! isset( $data['hash'] ) || ! isset( $data['trade_order_id'] ) ) {
			return false;
		}

		//自定义插件ID,请与支付请求时一致
		if ( isset( $data['plugins'] ) && $data['plugins'] != $this->plugins ) {
			return false;
		}

		$hash = self::generate_xh_hash( $data, $this->appsecret );
		if ( $data['hash'] != $hash ) {
			//签名验证失败
			return false;
		}

		if ( $data['status'] == 'OD' ) {
			return true;
		} else {
			return false;
		}
	}
}
