<?php

namespace HaoZiTeam\WPChatGPT\Inc;

use Exception;
use Generator;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

class YiYan {
	private Client $client;
	private $session_id = null;
	private $parent_chat_id;

	public function __construct( $cookie ) {
		$this->client = new Client( [
			'headers' => [
				'Cookie'       => $cookie,
				'Content-Type' => 'application/json;charset=UTF-8',
				'User-Agent'   => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
				'Sec-Fetch-User' => '?1',
				'Sec-Fetch-Mode' => 'navigate',
				'Sec-Fetch-Site' => 'none',
				'Sec-Ch-Ua-Platform' => 'Windows',
				'Sec-Ch-Ua-Mobile' => '?0',
				'Sec-Ch-Ua' => '"Chromium";v="112", "Google Chrome";v="112", "Not:A-Brand";v="99"',
			],
			'cookies' => true
		] );
	}

	private function parse_image( $html ) {
		preg_match( '/<img src="(.*?)" \/><br>/', $html, $matches );
		if ( $matches ) {
			return '![image](' . $matches[1] . ')  ' . "\n";
		} else {
			return $html;
		}
	}

	/**
	 * @throws Exception
	 */
	private function check_response( $resp ) {
		$resp = json_decode( $resp, true );
		if ( ! isset( $resp['code'] ) || $resp['code'] != 0 ) {
			throw new Exception( $resp['msg'] );
		}
	}

	/**
	 * @throws Exception
	 */
	private function get_sign() {
		try {
			$response = $this->client->get( "https://chatgpt-proxy.lss233.com/yiyan-api/acs" );
		} catch ( GuzzleException $e ) {
			throw new Exception( '获取Acs-Token错误: ' . $e->getMessage() );
		}

		return json_decode( $response->getBody()->getContents(), true )['acs'];
	}

	/**
	 * @throws GuzzleException
	 * @throws Exception
	 */
	public function delete_conversation( $session_id ) {
		$response = $this->client->post(
			"https://yiyan.baidu.com/eb/session/delete",
			[
				'json' => [
					"sessionId"  => $session_id,
					"timestamp"  => time() * 1000,
					"deviceType" => "pc"
				]
			]
		);

		if ( $response->getStatusCode() >= 400 ) {
			throw new Exception( '一言接口错误: ' . $response->getStatusCode() );
		}
	}

	/**
	 * @throws GuzzleException
	 * @throws Exception
	 */
	public function new_conversation( $prompt ) {
		$response = $this->client->post(
			"https://yiyan.baidu.com/eb/session/new",
			[
				'json'    => [
					"sessionName" => $prompt,
					"timestamp"   => time() * 1000,
					'plugins'     => [],
					"deviceType"  => "pc"
				],
				'headers' => [
					'Acs-Token' => $this->get_sign(),
					'Referer'   => 'https://yiyan.baidu.com/'
				]
			]
		);

		if ( $response->getStatusCode() >= 400 ) {
			throw new Exception( '一言接口错误: ' . $response->getStatusCode() );
		}

		$data = $response->getBody()->getContents();

		try {
			$this->check_response( $data );
		} catch ( Exception $e ) {
			throw new Exception( '一言接口错误: ' . $e->getMessage() );
		}
		$data                 = json_decode( $data, true );
		$this->session_id     = $data['data']['sessionId'];
		$this->parent_chat_id = 0;
	}

	/**
	 * @throws GuzzleException
	 * @throws Exception
	 */
	public function ask( $prompt, $session_id = "", $parent_chat_id = "" ): Generator {
		if ( empty( $session_id ) ) {
			$this->new_conversation( $prompt );
		} else {
			$this->session_id = $session_id;
		}
		if ( ! empty( $parent_chat_id ) ) {
			$this->parent_chat_id = $parent_chat_id;
		}

		try {
			$response = $this->client->post( 'https://yiyan.baidu.com/eb/chat/new', [
				'json'    => [
					'sessionId'    => $this->session_id,
					'text'         => $prompt,
					'parentChatId' => $this->parent_chat_id,
					'type'         => 10,
					'timestamp'    => time() * 1000,
					'deviceType'   => 'pc',
					'code'         => 0,
					'msg'          => '',
					'plugins'      => [],
					'pluginInfo'   => '',
					'jt'           => '',
					'sign'         => $this->get_sign(),
				],
				'headers' => [
					'Acs-Token' => $this->get_sign(),
					'Referer'   => 'https://yiyan.baidu.com/'
				]
			] );
		} catch ( GuzzleException $e ) {
			throw new Exception( '一言接口错误: ' . $e->getMessage() );
		}

		$data = $response->getBody()->getContents();

		try {
			$this->check_response( $data );
		} catch ( Exception $e ) {
			throw new Exception( '一言接口错误: ' . $e->getMessage() );
		}

		$data = json_decode( $data, true );

		$chat_id              = $data['data']['botChat']['id'];
		$chat_parent_id       = $data['data']['botChat']['parent'];
		$this->parent_chat_id = $chat_id;

		$sentence_id   = 0;
		$full_response = '';
		while ( true ) {
			$response = $this->client->post( 'https://yiyan.baidu.com/eb/chat/query', [
				'json'    => [
					'chatId'       => $chat_id,
					'parentChatId' => $chat_parent_id,
					'sentenceId'   => $sentence_id,
					'stop'         => 0,
					'timestamp'    => time() * 1000,
					'deviceType'   => 'pc',
					'sign'         => $this->get_sign()
				],
				'headers' => [
					'Acs-Token' => $this->get_sign(),
					'Referer'   => 'https://yiyan.baidu.com/'
				]
			] );
			$data     = $response->getBody()->getContents();
			try {
				$this->check_response( $data );
			} catch ( Exception $e ) {
				throw new Exception( '一言接口错误: ' . $e->getMessage() );
			}

			$data = json_decode( $data, true );

			$sentence_id = $data['data']['sent_id'];
			$content     = $data['data']['content'];

			if ( $content != '' ) {
				$content       = $this->parse_image( $content );
				$full_response .= $content;
				yield [
					'answer'         => $full_response,
					'now'            => $content,
					'session_id'     => $this->session_id,
					'parent_chat_id' => $this->parent_chat_id,
					'chat_id'        => $chat_id,
				];
			}

			usleep( 1000 );

			if ( $data['data']['is_end'] != 0 || $data['data']['stop'] != 0 ) {
				break;
			}
		}

		$this->parent_chat_id = $chat_id;
	}
}