<?php

namespace HaoZiTeam\WPChatGPT\Service;

use RAT;

defined( 'ABSPATH' ) || exit;

class Setting {

	private string $prefix = 'wp-chatgpt';

	public function __construct() {
		add_filter( 'rat_welcome_page', '__return_false' );
		add_filter( 'rat_fa4', '__return_true' );
		self::admin_init();
	}

	public function admin_init(): void {
		RAT::createOptions( $this->prefix, [
			'framework_title' => 'AI 聊天',
			'menu_title'      => 'AI 聊天',
			'menu_slug'       => 'wp-chatgpt',
			'menu_type'       => 'submenu',
			'menu_parent'     => 'options-general.php',
			'show_bar_menu'   => false,
			'show_sub_menu'   => false,
			'show_search'     => false,
			'footer_text'     => 'AI 聊天 设置页面',
			'theme'           => 'light',
			'enqueue_webfont' => false,
			'async_webfont'   => true,
		] );

		RAT::createSection( $this->prefix, [
			'title'  => 'AI 聊天 设置',
			'fields' => [
				[
					'type'    => 'content',
					'content' => '请新建页面并插入<code>[wp-chatgpt]</code>简码即可使用。若显示错位，你可以像这样添加class<code>[wp-chatgpt class="wp-chatgpt-frontend your-class"]</code>进行调整',
				],
				[
					'id'    => 'auth-id',
					'type'  => 'text',
					'title' => '授权ID',
					'desc'  => '输入购买的授权ID',
				],
				[
					'id'    => 'auth-key',
					'type'  => 'text',
					'title' => '授权KEY',
					'desc'  => '输入购买的授权KEY',
				],
				[
					'id'       => 'gpt-type',
					'type'     => 'radio',
					'title'    => '接口选择',
					'inline'   => true,
					'options'  => [
						'chatgpt-3.5' => 'API版',
						'chatgpt-web' => '网页版',
						'yiyan'       => '文心一言(Beta)',
					],
					'default'  => 'chatgpt-3.5',
					'subtitle' => '选择使用的 AI 接口',
					'desc'     => 'API版：最新的 API 版本，速度快且无限制（缺点：收费且限制会话长度）<br>网页版：体验与官网一致，免费且无会话限制（缺点：速度较慢且账号有速率限制）<br>文心一言：懂的都懂，不做介绍<br>注意从 API 版切换到其他版本将导致之前 API 版的会话记录无法继续！',
				],
				[
					'id'         => 'openai-api-server',
					'type'       => 'text',
					'title'      => 'API 服务器',
					'subtitle'   => 'AI API 接口地址',
					'desc'       => '不填则使用默认 API 服务器地址，如需使用其他 API 服务器，请填写完整的 API 地址。',
					'dependency' => [
						'gpt-type',
						'!=',
						'yiyan',
					],
				],
				[
					'id'         => 'chatgpt-system-prompt',
					'type'       => 'textarea',
					'default'    => '你是 ChatGPT，一个由 OpenAI 训练的大型语言模型，模型是 GPT-3.5，当前时间是 {datetime}。尽可能简洁地回答。',
					'title'      => 'AI 默认设定',
					'desc'       => 'API 版专属的设定词，将作为系统词伴随整个会话。可用变量：<code>{datetime}</code>（当前日期和时间）',
					'dependency' => [
						'gpt-type',
						'==',
						'chatgpt-3.5',
					],
				],
				[
					'id'           => 'gpt-avatar',
					'type'         => 'media',
					'title'        => 'AI 头像',
					'desc'         => 'AI 头像，不填则使用默认头像',
					'library'      => 'image',
					'button_title' => '选择头像',
					'remove_title' => '移除头像',
				],
				[
					'id'     => 'sensitive-words',
					'type'   => 'repeater',
					'title'  => '敏感词过滤',
					'desc'   => '输入敏感词，当检测到输出/输入中包含敏感词时，将会自动过滤',
					'fields' => [
						[
							'id'    => 'sensitive-word',
							'type'  => 'text',
							'title' => '敏感词',
						],
					],
				],
				[
					'id'      => 'baidu-censoring',
					'type'    => 'switcher',
					'default' => false,
					'title'   => '百度文本审核',
					'desc'    => '开启后，将会使用百度文本审核 API 对<b>用户输入</b>进行审核，若审核不通过将会自动过滤',
				],
				[
					'id'         => 'baidu-censoring-api-key',
					'type'       => 'text',
					'title'      => '百度文本审核 API KEY',
					'desc'       => '开启百度文本审核后，需要填写百度文本审核应用 API KEY',
					'dependency' => [
						'baidu-censoring',
						'==',
						'true',
					],
				],
				[
					'id'         => 'baidu-censoring-secret-key',
					'type'       => 'text',
					'title'      => '百度文本审核 SECRET KEY',
					'desc'       => '开启百度文本审核后，需要填写百度文本审核应用 SECRET KEY',
					'dependency' => [
						'baidu-censoring',
						'==',
						'true',
					],
				],
				[
					'id'    => 'no-login-notice',
					'type'  => 'text',
					'title' => '未登录提醒',
					'desc'  => '当访客未登录进入聊天时，将会提示此内容。不填默认为<code>请先登录</code>',
				],
				[
					'id'      => 'stream-output',
					'type'    => 'switcher',
					'default' => true,
					'title'   => '流式输出',
					'label'   => '开启后，将会在对话框中实时显示响应内容，而不是等待响应结束后一次性显示。大多数 CDN 不兼容此功能，如果对话经常 JSON 响应错误，请关闭此功能',
				],
				[
					'id'      => 'custom-conversation',
					'type'    => 'switcher',
					'default' => false,
					'title'   => '免登录会话',
					'label'   => '开启后，访客可以在不登录的情况下进行对话，会话记录将统一保存在一个指定的用户下',
				],
				[
					'id'         => 'custom-conversation-user',
					'type'       => 'number',
					'default'    => '1',
					'title'      => '免登录会话用户',
					'desc'       => '输入一个用户 ID，当访客使用免登录会话时，会话记录将统一保存在此用户下（不要使用管理员账号以免带来安全问题）',
					'dependency' => [
						'custom-conversation',
						'==',
						'true',
					],
				],
				[
					'id'      => 'conversation-trash',
					'type'    => 'switcher',
					'default' => false,
					'title'   => '对话回收站',
					'label'   => '开启后，用户删除的对话将会被移到回收站中，而不是直接删除',
				],
				[
					'id'      => 'message-trash',
					'type'    => 'switcher',
					'default' => false,
					'title'   => '消息回收站',
					'label'   => '开启后，用户删除的消息将会被移到回收站中，而不是直接删除',
				],
				[
					'id'      => 'fuck-nonce',
					'type'    => 'switcher',
					'default' => false,
					'title'   => '禁用nonce',
					'label'   => '如果有弹窗 Response 403 之类的，F12检查响应为cookie验证失败的情况可以打开此开关解决。注意这会导致跨站安全问题，插件工作正常请不要打开',
				],
				[
					'id'      => 'iframe-height',
					'type'    => 'text',
					'title'   => '容器高度',
					'desc'    => '设置插件前台容器的高度，默认<code>70vh</code>',
					'default' => '70vh'
				],
			],
		] );

		RAT::createSection( $this->prefix, [
			'title'  => 'AI 账号',
			'fields' => [
				[
					'type'    => 'content',
					'content' => '这里设置的账号，需要与前面选择的 AI 接口对应，否则将无法正常对话<br>如果使用网页版后端，账号设置后不建议随意修改，可能导致之前的会话记录无法继续',
				],
				[
					'id'                     => 'openai-accounts',
					'type'                   => 'group',
					'title'                  => '账号列表',
					'subtitle'               => '添加 AI 账号',
					'desc'                   => '添加多个 AI 账号，将会自动进行轮询，以降低账号被限制的风险',
					'button_title'           => '添加账号',
					'accordion_title_number' => true,
					'fields'                 => [
						[
							'id'    => 'account-note',
							'type'  => 'text',
							'title' => '账号备注',
							'desc'  => '账号备注，用于区分账号',
						],
						[
							'id'      => 'account-select',
							'type'    => 'radio',
							'title'   => '账号类型',
							'desc'    => '账号类型需要与前面选择的 AI 后端对应，否则将无法正常对话',
							'inline'  => true,
							'options' => [
								'chatgpt-3.5' => 'API版',
								'chatgpt-web' => '网页版',
								'yiyan'       => '一言',
							],
							'default' => 'chatgpt-3.5',
						],
						[
							'id'          => 'account-secret-key',
							'type'        => 'text',
							'title'       => 'Secret Key',
							'subtitle'    => 'AI 账号的 Secret Key',
							'desc'        => '登录后从 <a href="https://platform.openai.com/account/api-keys" target="_blank">https://platform.openai.com/account/api-keys</a> 获取',
							'placeholder' => '一串以sk-开头的东西',
							'dependency'  => [
								'account-select',
								'==',
								'chatgpt-3.5',
							],
						],
						[
							'id'          => 'account-access-token',
							'type'        => 'text',
							'title'       => 'Access Token',
							'subtitle'    => '有效期一个月，过期后需要重新获取',
							'desc'        => '登录后从 <a href="https://chat.openai.com/api/auth/session" target="_blank">https://chat.openai.com/api/auth/session</a> 获取',
							'placeholder' => '一串以ey开头的东西',
							'dependency'  => [
								'account-select',
								'==',
								'chatgpt-web',
							],
						],
						[
							'id'          => 'account-yiyan-cookie',
							'type'        => 'text',
							'title'       => '文心一言Cookie',
							'subtitle'    => '文心一言账号的 Cookie',
							'desc'        => '登录后从 <a href="https://yiyan.baidu.com/" target="_blank">https://yiyan.baidu.com/</a> 获取',
							'placeholder' => '一串很长的东西',
							'dependency'  => [
								'account-select',
								'==',
								'yiyan',
							],
						],
						[
							'id'         => 'chatgpt-web-model',
							'type'       => 'select',
							'title'      => '网页版模型',
							'subtitle'   => '选择一个网页版你想使用的模型',
							'desc'       => '对于免费账号，只能使用 gpt-3.5 模型，对于PLUS账号，可以使用 gpt-3.5-paid 和 gpt-4 模型',
							'options'    => [
								'text-davinci-002-render-sha'  => 'GPT-3.5',
								'gpt-4'                        => 'GPT-4 (需要PLUS账号)',
							],
							'default'    => 'text-davinci-002-render-sha',
							'dependency' => [
								'account-select',
								'==',
								'chatgpt-web',
							],
						],
						[
							'id'         => 'chatgpt-api-model',
							'type'       => 'select',
							'title'      => 'API版模型',
							'subtitle'   => '选择一个API版你想使用的模型',
							'desc'       => '如果账号没有 gpt-4 API权限，请不要选择 gpt-4 模型',
							'options'    => [
								'gpt-3.5-turbo' => 'GPT-3.5-Turbo',
								'gpt-3.5-turbo-16k' => 'GPT-3.5-Turbo-16K',
								'gpt-4'         => 'GPT-4 (账号需要绑卡)',
							],
							'default'    => 'gpt-3.5-turbo',
							'dependency' => [
								'account-select',
								'==',
								'chatgpt-3.5',
							],
						]
					],
				],
			],
		] );

		RAT::createSection( $this->prefix, [
			'title'  => '付费对话设置',
			'fields' => [
				[
					'id'       => 'pay-switch',
					'type'     => 'switcher',
					'title'    => '付费开关',
					'subtitle' => '付费对话的总开关',
					'label'    => '开启后，用户拥有对话额度，对话会消耗额度，当额度为 0 时，将无法与 AI 进行对话',
				],
				[
					'id'         => 'pay-wxpay-settings',
					'type'       => 'fieldset',
					'title'      => '微信',
					'subtitle'   => '此处设置微信支付的相关参数',
					'dependency' => [
						'pay-switch',
						'==',
						'true',
					],
					'fields'     => [
						[
							'id'    => 'switch',
							'type'  => 'switcher',
							'title' => '开关',
							'desc'  => '开启后，用户才可以使用微信支付充值套餐',
						],
						[
							'id'         => 'type',
							'type'       => 'radio',
							'title'      => '接口选择',
							'desc'       => '支持设置官方接口、易支付接口和讯虎支付（虎皮椒）接口',
							'options'    => [
								'official' => '官方接口',
								'epay'     => '易支付接口',
								'hupijiao' => '讯虎支付（虎皮椒）接口',
							],
							'default'    => 'official',
							'inline'     => true,
							'dependency' => [
								'switch',
								'==',
								'true',
							],
						],
						[
							'id'         => 'official-settings',
							'type'       => 'fieldset',
							'title'      => '官方接口设置',
							'subtitle'   => '使用官方接口进行微信支付时的设置',
							'dependency' => [
								'type|switch',
								'==|==',
								'official|true',
							],
							'fields'     => [
								[
									'id'      => 'interface-type',
									'type'    => 'radio',
									'title'   => '微信接口类型',
									'desc'    => '支持设置H5支付和扫码支付及自动模式（PC端扫码移动端H5）',
									'options' => [
										'wap'  => 'H5支付',
										'scan' => '扫码支付',
										'auto' => '自动选择',
									],
									'default' => 'wap',
									'inline'  => true,
								],
								[
									'id'    => 'mch-id',
									'type'  => 'text',
									'title' => 'ID',
									'desc'  => '微信支付商户号，可在 账户中心->商户信息 查看',
								],
								[
									'id'    => 'mch-secret-key',
									'type'  => 'text',
									'title' => '商户秘钥',
									'desc'  => '微信支付商户秘钥，32位形如md5值的字符串，可在 账户中心->API安全 中设置',
								],
								[
									'id'    => 'app-id',
									'type'  => 'text',
									'title' => '公众号 APPID',
									'desc'  => '公众号 APPID，可在 设置与开发->基本配置->开发者ID(AppID) 中查看',
								],
							],
						],
						[
							'id'         => 'epay-settings',
							'type'       => 'fieldset',
							'title'      => '易支付设置',
							'subtitle'   => '使用易支付进行微信支付，需要正确设置 API 地址、商户 ID、商户密钥',
							'dependency' => [
								'type|switch',
								'==|==',
								'epay|true',
							],
							'fields'     => [
								[
									'id'    => 'apiurl',
									'type'  => 'text',
									'title' => 'API 地址',
									'desc'  => '易支付 API 地址',
								],
								[
									'id'    => 'pid',
									'type'  => 'text',
									'title' => '商户 PID',
									'desc'  => '易支付商户 PID',
								],
								[
									'id'    => 'key',
									'type'  => 'text',
									'title' => '商户密钥',
									'desc'  => '易支付商户密钥',
								],
							],
						],
						[
							'id'         => 'hupijiao-settings',
							'type'       => 'fieldset',
							'title'      => '讯虎支付（虎皮椒）设置',
							'subtitle'   => '使用虎皮椒进行微信支付，需要正确设置 APP ID、APP Secret',
							'dependency' => [
								'type|switch',
								'==|==',
								'hupijiao|true',
							],
							'fields'     => [
								[
									'id'    => 'apiurl',
									'type'  => 'text',
									'title' => '支付网关',
									'desc'  => '讯虎支付（虎皮椒）支付网关，例如<code>https://api.xunhupay.com/payment/do.html</code>',
								],
								[
									'id'    => 'appid',
									'type'  => 'text',
									'title' => 'APP ID',
									'desc'  => '讯虎支付（虎皮椒） APP ID，在后台查看',
								],
								[
									'id'    => 'appsecret',
									'type'  => 'text',
									'title' => 'APP Secret',
									'desc'  => '讯虎支付（虎皮椒） APP Secret，在后台查看',
								],
							],
						],
					],
				],
				[
					'id'         => 'pay-alipay-settings',
					'type'       => 'fieldset',
					'title'      => '支付宝',
					'subtitle'   => '此处设置支付宝支付的相关参数',
					'dependency' => [
						'pay-switch',
						'==',
						'true',
					],
					'fields'     => [
						[
							'id'    => 'switch',
							'type'  => 'switcher',
							'title' => '开关',
							'desc'  => '开启后，用户才可以使用支付宝支付充值套餐',
						],
						[
							'id'         => 'type',
							'type'       => 'radio',
							'title'      => '接口选择',
							'desc'       => '支持设置官方接口、易支付接口和讯虎支付（虎皮椒）接口',
							'options'    => [
								'official' => '官方接口',
								'epay'     => '易支付接口',
								'hupijiao' => '讯虎支付（虎皮椒）接口',
							],
							'default'    => 'official',
							'inline'     => true,
							'dependency' => [
								'switch',
								'==',
								'true',
							],
						],
						[
							'id'         => 'official-settings',
							'type'       => 'fieldset',
							'title'      => '官方接口设置',
							'subtitle'   => '使用官方接口进行支付宝支付时的设置',
							'dependency' => [
								'type|switch',
								'==|==',
								'official|true',
							],
							'fields'     => [
								[
									'id'      => 'interface-type',
									'type'    => 'radio',
									'title'   => '支付宝接口类型',
									'desc'    => '只支持扫码支付（当面付）',
									'options' => [
										'scan' => '扫码支付（当面付）',
									],
									'default' => 'scan',
									'inline'  => true,
								],
								[
									'id'    => 'app-id',
									'type'  => 'text',
									'title' => 'APP ID',
									'desc'  => '支付宝分配的 APP ID',
								],
								[
									'id'    => 'private-key',
									'type'  => 'textarea',
									'title' => '应用私钥',
									'desc'  => '支付宝应用私钥证书，可在 应用详情->开发设置->接口加签方式 中设置',
								],
								[
									'id'    => 'ali-public-key',
									'type'  => 'textarea',
									'title' => '支付宝公钥',
									'desc'  => '支付宝公钥证书，设置应用私钥后，可下载得到',
								],
							],
						],
						[
							'id'         => 'epay-settings',
							'type'       => 'fieldset',
							'title'      => '易支付设置',
							'subtitle'   => '使用易支付进行支付宝支付，需要正确设置 API 地址、商户 ID、商户密钥',
							'dependency' => [
								'type|switch',
								'==|==',
								'epay|true',
							],
							'fields'     => [
								[
									'id'    => 'apiurl',
									'type'  => 'text',
									'title' => 'API 地址',
									'desc'  => '易支付 API 地址',
								],
								[
									'id'    => 'pid',
									'type'  => 'text',
									'title' => '商户 PID',
									'desc'  => '易支付商户 PID',
								],
								[
									'id'    => 'key',
									'type'  => 'text',
									'title' => '商户密钥',
									'desc'  => '易支付商户密钥',
								],
							],
						],
						[
							'id'         => 'hupijiao-settings',
							'type'       => 'fieldset',
							'title'      => '讯虎支付（虎皮椒）设置',
							'subtitle'   => '使用虎皮椒进行支付宝支付，需要正确设置 APP ID、APP Secret',
							'dependency' => [
								'type|switch',
								'==|==',
								'hupijiao|true',
							],
							'fields'     => [
								[
									'id'    => 'apiurl',
									'type'  => 'text',
									'title' => '支付网关',
									'desc'  => '讯虎支付（虎皮椒）支付网关，例如<code>https://api.xunhupay.com/payment/do.html</code>',
								],
								[
									'id'    => 'appid',
									'type'  => 'text',
									'title' => 'APP ID',
									'desc'  => '讯虎支付（虎皮椒） APP ID，在后台查看',
								],
								[
									'id'    => 'appsecret',
									'type'  => 'text',
									'title' => 'APP Secret',
									'desc'  => '讯虎支付（虎皮椒） APP Secret，在后台查看',
								],
							],
						],
					],
				],
				[
					'id'         => 'pay-return-url',
					'type'       => 'text',
					'title'      => '支付成功跳转地址',
					'subtitle'   => '设置支付成功后跳转的地址',
					'desc'       => '设置支付成功后跳转的地址，例如<code>https://www.example.com/aichat</code>，留空则跳转到首页，注意部分接口可能不支持<br>部分易支付由于回调数据<code>name</code>字段和WordPress机制冲突，可以设置一个中转页面解决',
					'dependency' => [
						'pay-switch',
						'==',
						'true',
					],
				],
				[
					'id'          => 'pay-new-user-quota',
					'type'        => 'number',
					'title'       => '新用户初始额度',
					'subtitle'    => '新用户初始额度，单位：次',
					'desc'        => '设置给新用户注册时赠送的免费额度，单位：次',
					'default'     => 10,
					'placeholder' => '10',
					'dependency'  => [
						'pay-switch',
						'==',
						'true',
					],
				],
				[
					'id'           => 'pay-packages',
					'type'         => 'group',
					'title'        => '套餐设置',
					'subtitle'     => '设置用户可以购买的额度套餐',
					'button_title' => '添加套餐',
					'fields'       => [
						[
							'id'    => 'package-name',
							'type'  => 'text',
							'title' => '套餐名称',
							'desc'  => '套餐名称，用于区分套餐',
						],
						[
							'id'    => 'package-price',
							'type'  => 'number',
							'title' => '套餐价格',
							'desc'  => '套餐价格，单位为元',
						],
						[
							'id'    => 'package-quota',
							'type'  => 'number',
							'title' => '套餐额度',
							'desc'  => '套餐额度，单位为次',
						],
					],
					'dependency'   => [
						'pay-switch',
						'==',
						'true',
					],
				],
			],
		] );
	}
}
